﻿using System.Globalization;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Converter.Models;
using CsvHelper.Configuration;
using CsvHelper;
using Newtonsoft.Json;
using YamlDotNet.Serialization;


namespace Converter.FileRead
{
    public class ReadData
    {
        public static void ReadFileData<T>()
        {
            Console.WriteLine("File name: ");
            string fileName = Console.ReadLine();
            string fileExtension = Path.GetExtension(fileName).ToLower();
            Console.WriteLine();

            switch (fileExtension)
            {
                case ".csv":
                    ReadCsv<T>(fileName);
                    break;
                case ".json":
                    ReadJson<T>(fileName);
                    break;
                case ".xml":
                    ReadXml<T>(fileName);
                    break;
                case ".yaml":
                case ".yml":
                    ReadYaml<T>(fileName);
                    break;
                default:
                    Console.WriteLine("Unsupported file format!");
                    break;
            }
        }

        public static void ReadCsv<T>(string fileName)
        {
            var configuration = new CsvConfiguration(CultureInfo.InvariantCulture)
            {
                Encoding = Encoding.UTF8,
                Delimiter = ","
            };

            using (var fs = File.Open(fileName, FileMode.Open, FileAccess.Read, FileShare.Read))
            {
                using (var textReader = new StreamReader(fs, Encoding.UTF8))
                using (var csv = new CsvReader(textReader, configuration))
                {
                    var data = csv.GetRecords<T>();

                    foreach (var element in data)
                    {
                        Console.WriteLine(element);
                    }
                }
            }
        }

        public static void ReadJson<T>(string fileName)
        {
            string jsonContent = File.ReadAllText(fileName);
            List<T> jsonModels = JsonConvert.DeserializeObject<List<T>>(jsonContent);

            foreach (var element in jsonModels)
            {
                Console.WriteLine(element);
            }
        }

        public static void ReadXml<T>(string fileName)
        {
            using (var reader = XmlReader.Create(fileName))
            {
                var serializer = new XmlSerializer(typeof(List<T>));
                var xmlModels = (List<T>)serializer.Deserialize(reader);

                foreach (var element in xmlModels)
                {
                    Console.WriteLine(element);
                }
            }
        }

        public static void ReadYaml<T>(string fileName)
        {
            string yamlContent = File.ReadAllText(fileName);
            var deserializer = new DeserializerBuilder().Build();
            var yamlModels = deserializer.Deserialize<List<T>>(yamlContent);

            foreach (var element in yamlModels)
            {
                Console.WriteLine(element);
            }
        }

    }
}
