﻿using CsvHelper;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using YamlDotNet.Serialization;

namespace Converter.FileOperations
{
    public class WriteData
    {
        public static void WriteFileData<T>(List<T> data)
        {
            Console.WriteLine("File name: ");
            string fileName = Console.ReadLine();
            Console.WriteLine();

            string fileExtension = Path.GetExtension(fileName).ToLower();
            Console.WriteLine();

            switch (fileExtension)
            {
                case ".xml":
                    WriteToXml(data, fileName);
                    break;
                case ".json":
                    WriteToJson(data, fileName);
                    break;
                case ".yaml":
                case ".yml":
                    WriteToYaml(data, fileName);
                    break;
                case ".csv":
                    WriteToCsv(data, fileName);
                    break;
                default:
                    Console.WriteLine("Unsupported file format!");
                    break;
            }
        }

        private static void WriteToXml<T>(List<T> data, string fileName)
        {
            using (var writer = new StreamWriter(fileName))
            {
                var serializer = new System.Xml.Serialization.XmlSerializer(typeof(List<T>));
                serializer.Serialize(writer, data);
            }
            Console.WriteLine($"Data successfully written to {fileName}");
        }

        private static void WriteToJson<T>(List<T> data, string fileName)
        {
            string json = JsonConvert.SerializeObject(data, Formatting.Indented);
            File.WriteAllText(fileName, json);
            Console.WriteLine($"Data successfully written to {fileName}");
        }

        private static void WriteToYaml<T>(List<T> data, string fileName)
        {
            var serializer = new SerializerBuilder().Build();
            string yaml = serializer.Serialize(data);
            File.WriteAllText(fileName, yaml);
            Console.WriteLine($"Data successfully written to {fileName}");
        }

        private static void WriteToCsv<T>(List<T> data, string fileName)
        {
            var config = new CsvHelper.Configuration.CsvConfiguration(System.Globalization.CultureInfo.InvariantCulture)
            {
                HasHeaderRecord = true,
                Delimiter = ",",
                IgnoreBlankLines = true
            };

            using (var writer = new StreamWriter(fileName))
            using (var csv = new CsvWriter(writer, config))
            {
                csv.WriteRecords(data);
            }
            Console.WriteLine($"Data successfully written to {fileName}");
        }
    }
}
